#ifndef _L1DATA
#define _L1DATA

#include <stdio.h>

/***
 *
 * This pretty well mirrors the FLT message as defined in "L1SIMU user guide" Oct 25, 1996
 *
 ***/
  
typedef enum {
  MUMU,
  EE,
  EMU,
  HIPT_MU,
  HIPT_E,
  PIPI,
  EE_JPSI,
  EE_UPSILON,
  N_TRIG_TYPES
} TRIGTYPE;

typedef enum {
  EPLUS,
  EMINUS,
  MUPLUS,
  MUMINUS,
  PIPLUS,
  PIMINUS
} PARTTYPE;

struct L2FR_L1Data {
  int eventNumber;       /* event number */
  int bx;                /* trigger bunch crossing number */
  int nTriggers;         /* number of triggers */
  int nTracks;           /* number of tracks */
};
typedef struct L2FR_L1Data L2FR_L1Data;

struct L2FR_L1Trig {
  TRIGTYPE trigID;       /* trigger type */
  int trk1;              /* first trigger track number */
  int trk2;              /* second trigger track number */
};
typedef struct  L2FR_L1Trig L2FR_L1Trig;


struct L2FR_L1Track {
  PARTTYPE trackID;      /* pretrigger track ID */
  float energy;          /* total energy reconstructed */
  float pxpz;            /* Px/Pz */
  float pypz;            /* Py/pz */
  int charge;            /* charge */
  float xtc2;            /* x position of the track in TC2 SL */
  float sx2;             /* hit width in TC2 SL (+- 0.5sx) */
  float ytc2;            /* y position of the track in TC2 SL */
  float ztc2;            /* z position of the track in TC2 SL */
  float xpc1;            /* x position of the track in PC1 SL */
  float sx1;             /* hit width in PC1 SL (+- 0.5sx) */
  float zpc1;            /* z position of the track in PC1 SL */
  float y2_err;          /* error at tc2...a kluge until we know how to compute it */
};
typedef struct L2FR_L1Track L2FR_L1Track;



L2FR_L1Data *l2fr_L1DataLookup( int );
L2FR_L1Data *l2fr_L1DataIterateInit();
L2FR_L1Data *l2fr_L1DataIterateNext();
L2FR_L1Data *l2fr_L1DataRead(FILE *);

void l2fr_L1DataInsert(L2FR_L1Data *);
void l2fr_L1DataPrintHeader(FILE *);
void l2fr_L1DataPrint(FILE *);
void l2fr_L1DataPrintData(FILE *, L2FR_L1Data *);
void l2fr_L1DataReadHeader(FILE *);
void l2fr_L1DataClear();
#
#endif
