#ifndef EVENT_FLG
#define EVENT_FLG

#ifndef SLTINFO

#include "l2defs.h"
#include "oneone.h"
#include "l2geo.h"
#include "tgs.h"
#endif


#define L2MAX_TRKS_TRIG     2    /* Max # of tracks per trigger   */
#define L2MAX_TRIG_TRK     30    /* Max # of triggers per track   */
#define L2MAX_TRACKS       25    /* Max # of SLT tracks per event */

#define N_XBACK_LAYERS 6    /* number of superlayers behind magnet */

typedef enum {
  L2EVENT_REJECT,
  L2EVENT_NOT_FINISHED,
  L2EVENT_ACCEPT,
  L2EVENT_FORCED
} L2_EVENT_STAT;

typedef enum {
  L2ALGO_IN_INIT,
  L2ALGO_IN_SLICER,
  L2ALGO_IN_REFITX,
  L2ALGO_IN_REFITY,
  L2ALGO_IN_MAGNET,
  L2ALGO_IN_VERTEX,
  L2ALGO_IN_TARGET,
  L2ALGO_N_ALGO_SECTIONS
} L2ALGOSTAT;

typedef enum {
  L2TRIGGER_REJECT, 
  L2TRIGGER_NOT_FINISHED,
  L2TRIGGER_ACCEPT
} L2TRIGSTAT;

typedef enum {
  L2TRACK_REJECT, 
  L2TRACK_NOT_FINISHED,
  L2TRACK_ACCEPT,
  L2TRACK_CANCELED,
  L2TRACK_WEAK_ACCEPT
} L2TRACKSTAT;

typedef enum { 
  L2TKINFO_NO_TRIG_ASSOC,
  L2TKINFO_TRIG_ASSOC,
  L2TKINFO_HIT_COUNT_BAD,
  L2TKINFO_HIT_COUNT_OK,
  L2TKINFO_SLICER_BAD,
  L2TKINFO_SLICER_OK,
  L2TKINFO_REFITX_BAD,
  L2TKINFO_REFITX_OK,
  L2TKINFO_REFITY_BAD,
  L2TKINFO_REFITY_OK,
  L2TKINFO_MAGNET_BAD,
  L2TKINFO_MAGNET_OK,
  L2TKINFO_SILICON_BAD,
  L2TKINFO_SILICON_OK
} L2TKINFO;

typedef enum {
  DIMUON, 
  DIELECTRON
} L2TRIGTYPE;

typedef enum {
  L2SILISUB_BAD,
  L2SILISUB_GOOD
} L2SILISUBSTAT;

/* Trigger flags set by the l2vertex code: */

#define L2TRIGFLAG_VERTEX 1            /* Two tracks come from common vertex */
#define L2TRIGFLAG_DETACHEDVERTEX 2    /* Vertex is not from a wire */
#define L2TRIGFLAG_COMMONWIRE 16       /* Tracks is from common wire */
#define L2TRIGFLAG_WIREMISSING1 32      /* Tracks is from common wire, but track 1 does not hit the wire */
#define L2TRIGFLAG_WIREMISSING2 64      /* Tracks is from common wire, but track 2 does not hit the wire */


/***************************************************************
 *
 * hits and hit lists
 *
 ***/

struct L2HIT { 
  int plane;
  int sl_rel;   /*PC1,MS10 = 0, PC2,MS11 = 1....TC2*/
  float rot;    /* hit coordinate */
  float offset; /* drift distance (or 0) */
  float res;
  float z;
  PLANE *pl;  /* pointer to associated plane geometry record */
};

typedef struct L2HIT L2HIT;   



/***************************************************************
 *
 * slicer stuff
 *
 ***/

/* 
   The name of this structure etc. should be changed...it is used only for
   x-back hits.
*/

#define L2MAX_HITS  200     /* Max # of hits per track in x-back */

struct L2HitSet {
   int nHits;                     /* # of hits used by the track */
   L2HIT Hit[L2MAX_HITS];         /* Hits belonging to this track */
};
typedef struct L2HitSet L2HITSET;


#define L2MAX_SLICER_TRACKS 10 /* Max # of Slicer tracks per FLT track */

#define SLICER_X1    8      /* Num. of divisions on X1 side */
#define SLICER_X2    8      /* Num. of divisions on X2 side */

struct L2SLICE {
  int nHitsCounted;       /* Number of hits already counted */
  int SLHits[N_XBACK_LAYERS];
  int nHits[SLICER_X1][SLICER_X2];
};
typedef struct L2SLICE L2SLICE;

/* The track structures as generated by each of the algortihm sections */

/** L2ROI is now an FLT_TRACK which is defined in flte.h **/

typedef FLT_TRACK L2ROI;

struct L2SLICERTRKSET {
  int nTracks;
  L2ROI l2SlicerTrk[L2MAX_SLICER_TRACKS];
};
typedef struct L2SLICERTRKSET L2SLICERTRKSET;



/***************************************************************
 *
 * Definitions for Refit
 *
 ***/

#define L2MAX_YB_HITS  400     /* Max # of hits per track in stereo-back */

struct L2_YBHITSET {
   int nHits;                     /* # of hits used by the track */
   L2HIT Hit[L2MAX_YB_HITS];      /* Hits belonging to this track */
};
typedef struct L2_YBHITSET L2_YBHITSET;


#define L2MAX_REFITTRKS 10     /* Max # of ReFit track per track */

#define REFIT_MAX_SUBTRK 5     /* Max # of Refit sub-tracks */
#define REFIT_MAX_HIT 50       /* Max # of hits used in Refit */

struct L2ReFitSubTrk {   /* ReFit Sub-track */
  int TrkNumber;         /* Refit track number */
  int nlayers;           /* # of single layers */
  int nEmptyRoI;         /* # of empty layers */
  int nUsed;             /* # of hits used */
  int nHoles;            /* # of layers not used */
  int nHolesInSeries;    /* # of holes in series */
  float chi2sum;         /* pseudo chisq */
  float quality;         /* track quality */
  float intercept[2];    /* track intercept */
  float slope[2];        /* track slope */
  float xpos;            /* track's present x position */
  float ypos;            /* track's present y position */
  float zpos;            /* track's present z position */
  float x_error;         /* track's present x position error*/
  float y_error;         /* track's present y position error */
  double sw[2];           /* sum ( 1 / err_i^2 ) */
  double swz[2];          /* sum ( z_i / err_i^2 ) */
  double swzz[2];         /* sum ( z_i * z_i / err_i^2 ) */
  double swx[2];          /* sum ( x_i / err_i^2 ) */
  double swxz[2];         /* sum ( x_i * z_i / err_i^2 ) */
  float delx;
  int nHits;
  L2HIT *pHit[REFIT_MAX_HIT];
};
typedef struct L2ReFitSubTrk L2REFITSUBTRK;

struct L2ReFitTrk {      /* ReFit track */
  int TrkNumber;         /* Refit track number */
  int nTracks;           /* # of sub tracks */
  L2ROI *parent;         /* pointer to parent slicer track */

  float richradius;          /* Rich information */
  float richtx;              /* from analysis package */
  float richty;

  L2REFITSUBTRK RefitSubTrk[REFIT_MAX_SUBTRK]; 
};
typedef struct L2ReFitTrk L2REFITTRK;

struct L2ReFitTrkSet {
   int nTracks;            /* # of track in the container */
   L2REFITTRK l2refittrk[L2MAX_REFITTRKS];
};
typedef struct L2ReFitTrkSet L2REFITTRKSET;



/***************************************************************
 *
 * Definitions for the magnet traversal
 *
 ***/

#define L2MAX_MAGNETTRKS 10    /* L2max_refittrks*refit_max_subtrks */   
                               /* Max # of Magnet tracks per FLT track */
#define MAGNET_MAX_SUBTRK 7    /* Max # of Magnet sub-tracks */
#define MAGNET_MAX_HIT   50    /* Max # of hits used by Magnet subtrack */
#define L2MAX_MAG_HITS   80    /* Max # of hits/track in raw hit array */

struct L2MagHitSet {
   int nHits;                     /* # of hits used by the track */
   L2HIT Hit[L2MAX_MAG_HITS];         /* Hits belonging to this track */
};
typedef struct L2MagHitSet L2_MAGHITSET;


struct L2MagnetSubTrk {   /* Magnet subtrack */
  int TrkNumber;          /* Magnet track number */
  float slopexz;          /* track's slope xz plane*/
  float slopeyz;          /* track's slope yz plane*/
  float invmom;           /* track's inverse momentum (1/pz)*charge */
  float xpos;             /* track's x position */
  float ypos;             /* track's y position */
  double error[5][5];      /* parameters error matrix */
  float zpos;             /* track's z position */
  float chisq;            /* current chisquare estimate */
  int nHits;
  L2HIT *pHit[MAGNET_MAX_HIT];
};
typedef struct L2MagnetSubTrk L2MAGNETSUBTRK;

 
 struct L2MagnetTrk    {                 /* Magnet track */
   int TrkNumber;                        /* Magnet track number */
   int Refpar;                           /* Refit parent */
   int RefSubpar;                        /* Refit subparent */
   int nTracks;                          /* n. of subtracks */
   L2MAGNETSUBTRK L2MagnetSubTrk[MAGNET_MAX_SUBTRK];
};
typedef struct L2MagnetTrk L2MAGNETTRK;


struct L2MagnetTrkSet {
  int nTracks;            /* # of tracks in the container = n.trks slicer X n. refit subtrks  */
  L2MAGNETTRK L2magnetTrk[L2MAX_MAGNETTRKS]; /* # of ROIs .le. nTrks (ovlap)*/
};
typedef struct L2MagnetTrkSet L2MAGNETTRKSET;


/***************************************************************
 *
 * Definitions for the silicon tracker
 *
 ***/

#define L2MAX_SILI_HITS   400     /* Max # of hits per L2TRACK track in vertex detector */
#define L2MAX_SILITRKS     10
#define SILI_MAX_HIT       32     /* the maximum number of hits on a subtrack */
#define L2SILI_MAX_SUBTRK  10



struct L2_SILIHITSET {
   int nHits;                     /* # of hits used by the track */
   L2HIT Hit[L2MAX_SILI_HITS];      /* Hits belonging to this track */
};
typedef struct L2_SILIHITSET L2_SILIHITSET;

struct L2SILIROI {
  int eventNumber;
  int trackIndex;
  int siliTrackIndex;
  float x1;              /* The RoI definition: */
  float x1_err;
  float y1;
  float y1_err;
  float z1;
  float x2;
  float x2_err;
  float y2;
  float y2_err;
  float z2;
};
typedef struct L2SILIROI L2SILIROI;

struct L2SiliSubTrk { /* Silicon track */
  float MOM   ;    /*  momemtum of track candidate */
  float SLO[2];    /*  current slope estimate */
  float INT[2];    /*  current intercept estimate */
  float S0[2];     /*  sum of weights */
  float SZ[2];     /*  sum of weighted z */
  float SZSQ[2];   /*  sum of weighted z**2 */
  float RX[2];     /*  sum of weighted x */
  float RXZ[2];    /*  sum of weighted x*z */
  float OS[2];     /*  sum of weigthed opposite direction cosines */
  float OZ[2];     /*  sum of weigthed z * opposite direction cosines */
  float OZSQ[2];   /*  sum of weighted z**2 * opposite direction cosines */
  int   NHIT[2];   /*  number of hits on track */
  int   NLTR[2];   /*  number of layers traversed in the last super plane */
  float CHI[2];    /*  current chisq estimate */
  

  L2SILIROI SiliROI;        /* ROI for this subtrack. Only used internally
                               in the silicon code to exclude hits. Redefined
                               from superlayer to superlayer. */

  L2SILISUBSTAT status;      /* Good or bad track? */
  short wirenr;              /* Assigned to this wire */
  float impact;              /* Impact parameter to wire */
  float alongwire;           /* distance along the the wire */

  int nClustsBrems;          /* How many clusters did we find? */

  float Ebrems;              /* Bremsstrahlung information from the */
  float xbrems;              /* analysis package */
  float ybrems;



  int nHits;
  L2HIT *pHit[SILI_MAX_HIT];

};
typedef struct L2SiliSubTrk L2SILISUBTRK;



struct L2SiliTrk {      /* Silicon track */
  int TrkNumber;               /* Silicon track number */
  int nTracks;                 /* # of sub tracks */
  L2MAGNETTRK    *pMagnetTrk;  /* Coresponding magnet track */
  L2MAGNETSUBTRK *pMagSubTrk;  /* pointer to initial slope and intercept from the magnet */

  L2SILISUBTRK SiliSubTrk[L2SILI_MAX_SUBTRK]; 
};
typedef struct L2SiliTrk L2SILITRK;

struct L2SiliTrkSet { /* Silicon track */
  int nTracks;        /* # of track in the container */
  L2SILITRK l2silitrk[L2MAX_SILITRKS];
};
typedef struct L2SiliTrkSet L2SILITRKSET;


/***************************************************************
 *
 * Vertex structure
 *
 ***/

struct L2Vertex {
  float chisq;                      /* Chi square of vertex fit */
  float xv;                         /* Fitted Vertex position */
  float yv;
  float zv;
  float sigxv;                      /* Errors on fitted vertex positions */
  float sigyv;
  float sigzv;
  float dxy;                        /* Distance between tracks */
  short wire;                         /* "Closest" wire to the vertex */
  float dz;                         /* z-distance from vertex to wire */
  float impact;                     /* distance to wire in x-y plane perpendicular to the wire */
  float alongwire;                  /* distance along the the wire */
  L2SILISUBTRK * trks[2];             /* The two tracks, from which the vertex was fitted */
};
typedef struct L2Vertex L2VERTEX;

/***************************************************************
 *
 * Trigger, track and event structures
 *
 ***/

struct L2Trigger {
  int TriggerNumber;                /* Trigger number */
  L2TRIGSTAT status;                /* Trigger status */
  L2TRIGTYPE trigID;                /* Trigger type */
  int nTrks;                        /* # of tracks belonging to this trigger */
  int TrkIndex[L2MAX_TRKS_TRIG];    /* index of tracks for this trigger */
  int flag;                         /* Trigger flags set by vertex (and others?) */
  L2VERTEX vertex;                  /* Most detached vertex of accepted vertices */
};
typedef struct L2Trigger L2TRIGGER;

struct L2Track {
  int TrkNumber;                    /* Track number */
  int FLTNumber;                    /* The corresponding FLT track number */
  L2TRACKSTAT status;               /* Track status */
  L2TKINFO info;                    /* More detailed track status */
  int nOutstanding;                 /* total # of outstanding messages for this track */

  L2HITSET      TrkHitSet;          /* x-back hits container */
  L2_YBHITSET   YBackHits;          /* stereo-back, behind magnet (all this will change!) */
  L2_MAGHITSET  MagHits;            /* Hit set for chambers in magnet */
  L2_SILIHITSET SiliHits;           /* Hit set for vertex dectector hits */

  L2ROI FLTTrk;                     /* FLT track */
  L2SLICE  slice;                   /* Slicer RoI/subRoI structure */

  L2SLICERTRKSET L2SlicerTrkSet;    /* Slicer track container */
  L2REFITTRKSET  ReFitTrkSet;       /* Refit track container */
  L2MAGNETTRKSET MagnetTrkSet;      /* Magnet track container */
  L2SILITRKSET   SiliTrkSet;        /* Silicon track container */

  int nTriggers;                    /* # of triggers using this track */
  int TriggerIndex[L2MAX_TRIG_TRK]; /* Index of triggers using this track */
};
typedef struct L2Track L2TRACK;

struct L2EVENT {
  int eventNumber;                       /* event number */
  int bxid;                              /* bunch crossing id */
  L2_EVENT_STAT status;                  /* event status */
  unsigned int eventType;                /* to be defined. e.g. FLT, ECAL_PRE, pulser...  */
  L2ALGOSTAT algo;                       /* which part of the SLT executing */
  int nOutstanding;                      /* total # of outstanding messages for the event */ 
  int lastTrkServiced;                   /* Track index serviced */
  int nTriggers;                         /* # of triggers in the event */
  int nFLTTracks;                        /* # of FLT tracks in this event */
  int nTracks;                           /* # of SLT tracks == # trigger-associated FLT tracks */
  int SLTTrack[TGS_MAX_TRACKS+1];        /* SLT trk num from FLT trk num or -1 if non-existent */
  L2TRIGGER Triggers[TGS_MAX_TRIGGERS];  /* Trigger bank */
  L2TRACK Tracks[L2MAX_TRACKS];          /* Track bank */
                                         /* the following fields are filled at event build time */ 
  unsigned int acceptMask;               /* event status (accept/reject) for each FLT trigger   */
  unsigned int trigMask;                 /* which trigger types are satisfied by any trigger    */
  unsigned int compMask;                 /* info available in rebuilt event                     */
  unsigned int procNode;                 /* processor number on which SLT code was run          */
  L2GEO_VERSION geo;                     /* Geometry set identifiers                            */
  long time;                             /* As returned by time(), filled during event build    */
};
typedef struct L2EVENT L2EVENT;


/***************************************************************
 *
 * Function declarations
 *
 ***/

int      L2Event_Init(void); /* begin run initializations */

L2EVENT *EventCreate(int event);
void EventPrint(FILE *fdout, const L2EVENT *pEvent);
void EventCountHitReplys(L2EVENT *, int);
void EventUpdateStatus(L2EVENT *pEvent);
void EventResetStatus(L2EVENT *pEvent, L2ALGOSTAT stat);
void EventShipOutTheGoodies(L2EVENT *pEvent);

int  Event_Getbxid( L2EVENT *ev );

void Event_TidyUp(L2EVENT *pEvent, unsigned int compMask);
/* ============================= Mogens: begin ================================= */
int  Event_InitEventHdr(L2EVENT *pEvent, unsigned int compMask);
int  Event_InitTrigBank(L2EVENT *pEvent, unsigned int compMask);
int  Event_InitTrackBank(L2EVENT *pEvent, unsigned int compMask);
int  Event_BuildEventHdr(L2EVENT *pEvent, char *buff, int *len, 
			 int maxlen, int *compid, int *rev);   /* Get L2 event top record */
int  Event_BuildEventAux(L2EVENT *pEvent, char *buff, int *len, 
			 int maxlen, int *compid, int *rev);   /* Get L2 event auxiliary record */
int  Event_GimeNextTrigBank(L2EVENT *pEvent, char *buff, int *len,
			    int maxlen, int *compid, int *rev);
int  Event_GimeNextTrackBank(L2EVENT *pEvent, char *buff, int *len,
			     int maxlen, int *compid, int *rev);

        /* Return pointer to L2TRIGGER bank # inst */
L2TRIGGER *Event_Get_L2TRIGGER (L2EVENT *pEvent, const int inst );
        /* Return pointer to L2TRACK bank # inst */
L2TRACK   *Event_Get_L2TRACK (L2EVENT *pEvent, const int inst );
/* ============================= Mogens:  end  ================================= */



void       TriggerPrint(FILE *fdout, const L2TRIGGER *pTrigger);
void       UpdateTriggerStatus(L2EVENT *pEvent, L2TRIGGER *pTrigger);
L2TRIGSTAT CheckTriggerStatus(L2EVENT *pEvent, L2TRIGGER *pTrigger);

void     TrackPrint(FILE *fdout, const L2TRACK *pTrack);
void     FLTTrackPrint(FILE *fdout, const L2ROI *roi);
void     UpdateTrackStatus(L2EVENT *pEvent, L2TRACK *pTrk);

const L2EVENT  *GetL2Event(int FCSevt); 

#endif
