#ifndef _L2IO
#define _L2IO

#include "event.h"      /* online event header */
#include "component.h"  /* online component definitions */
#include "l2event.h"
#include "etb.h"
#ifndef SLTINFO
#include "l2targ.h"
#endif

/* ******************* PLEASE NOTE *********************
   Current implentation of byteswapping in Event_* routines only
   work on integer and float, not double or short int!!!!
   ***************************************************** */


enum L2COMP_ID {   /* If you add anything to this list, put it at the end. */
  L2COMP_EVENT,
  L2COMP_EVENT_AUX,
  L2COMP_TRIG,
  L2COMP_TRACK,
  L2COMP_ECAL,          /* Not used - included in TRACK component */
  L2COMP_SLICER,
  L2COMP_SLICER_HIT,
  L2COMP_REFIT,
  L2COMP_REFIT_HIT,
  L2COMP_MAGNET,
  L2COMP_MAGNET_HIT,
  L2COMP_SILI,
  L2COMP_SILI_HIT,
  L2COMP_VERTEX        /* Not implemented yet */
};

enum L2IO_RETURNS {         /* return codes */
  L2IO_SUCCESS,
  L2IO_DONE,
  L2IO_NOT_FOUND,
  L2IO_TOO_BIG,
  L2IO_ERROR
};

#define L2IO_MAGIC 0x12345678

int L2IO_InstallAddDataHdls();


/*-------------------------   SLT Information   ---------------------------------*/

#define L2IO_EVENT_VERS     0
#define L2IO_EVENT_AUX_VERS 0
#define L2IO_TRIG_VERS      0
#define L2IO_TRACK_VERS     0
#define L2IO_SLICER_VERS     0
#define L2IO_REFIT_VERS     0
#define L2IO_MAGNET_VERS     0
#define L2IO_SILI_VERS     0
#define L2IO_SLICER_HIT_VERS     0
#define L2IO_REFIT_HIT_VERS     0
#define L2IO_MAGNET_HIT_VERS     0
#define L2IO_SILI_HIT_VERS     0

#define L2IO_REFIT_VERS2     1
#define L2IO_SILI_VERS2     1
#define L2IO_TRIG_VERS2      1


#define L2IO_FED_RECORD_ID 0x1000
#define L2IO_SLB_RECORD_ID 0x2000



struct L2IO_EVENT {
  unsigned int magic;      /* to check the endianess (is it a word? maybe endianitude)  */
  unsigned int eventType;  /* to be defined. e.g. FLT, ECAL_PRE, pulser...              */
  unsigned int procNode;   /* processor number on which SLT code was run                */
  unsigned int nTriggers;  /* number of FLT triggers                                    */
  unsigned int nTracks;    /* number of SLT tracks = #trigger-associated FLT tracks     */
  L2ALGOSTAT algo;         /* last algorithm section which executed                     */
  unsigned int acceptMask; /* event status (accept/reject) for each FLT trigger         */
  unsigned int trigMask;   /* which trigger types are satisfied by any trigger          */
  unsigned int l2compMask; /* mask of L2 sub components in event record                 */
  long time;               /* As returned by time(), filled during event build          */
                           /* Note: I use long instead of time_t, assuming linux writes */
};
typedef struct L2IO_EVENT L2IO_EVENT;

/*** 

     Auxilliary bank for event-related stuff which is needed if more than 
     just the top level event block is output. The idea is to keep only 
     summary information in the top block.

***/

struct L2IO_EVENT_AUX {
  L2GEO_VERSION geo;             /* Geometry set identifiers                            */
  unsigned int nFLTTracks;       /* # of FLT tracks in this event                       */
  int SLTTrack[TGS_MAX_TRACKS]; /* SLT trk num from FLT trk num or -1 if non-existent  */
                                 /* Here FLT tracks are numbered starting at 0 !!!      */
};
typedef struct L2IO_EVENT_AUX L2IO_EVENT_AUX;

struct L2IO_TRIG {
  unsigned int TriggerNumber;    /* Trigger number                   */
  L2TRIGSTAT status;             /* Trigger status                   */
  L2TRIGTYPE trigID;             /* Trigger type                     */
  unsigned int nTrks;            /* # of tracks belonging to this trigger  */
  int TrkIndex[L2MAX_TRKS_TRIG]; /* index of tracks for this trigger     */
};
typedef struct L2IO_TRIG L2IO_TRIG;

struct L2IO_TRIG2 {
  unsigned int TriggerNumber;    /* Trigger number                   */
  L2TRIGSTAT status;             /* Trigger status                   */
  L2TRIGTYPE trigID;             /* Trigger type                     */
  int flag;                      /* Trigger flags set by vertex      */
  float chisq;                   /* Chi square of vertex fit */
  float xv;                      /* Fitted Vertex position */
  float yv;
  float zv;
  float sigxv;                   /* Errors on fitted vertex positions */
  float sigyv;
  float sigzv;
  float dxy;                     /* Distance between tracks */
  int wire;                      /* "Closest" wire to the vertex */
  float dz;                      /* z-distance from vertex to wire */
  float impact;                  /* distance to wire in x-y plane perpendicular to the wire */
  float alongwire;               /* distance along the the wire */
  int trks[2];                   /* The two sili tracks, from which the vertex was fitted */
  int subtrks[2];                /* The two sili subtracks, from which the vertex was fitted */

  unsigned int nTrks;            /* # of tracks belonging to this trigger  */
  int TrkIndex[L2MAX_TRKS_TRIG]; /* index of tracks for this trigger     */
};
typedef struct L2IO_TRIG2 L2IO_TRIG2;

struct L2IO_TRACK {
  unsigned int TrkNumber; /* Track number                                      */
  unsigned int FLTNumber; /* The corresponding FLT track number                */
  L2TRACKSTAT status;     /* Track status                                      */
  L2TKINFO info;          /* More detailed track status                        */

  PARTTYPE FLTtrackID;    /* pretrigger track ID */
  float    FLTenergy;     /* FLT total energy reconstructed */
  float    FLTe_err;      /* error on FLT energy, or 0. if unavailable */
  float    FLTpxpz;       /* FLT Px/Pz */
  int      FLTcharge;     /* FLT charge */
  float    FLTx1;         /* The FLT RoI definition: */
  float    FLTx1_err;
  float    FLTy1;
  float    FLTy1_err;
  float    FLTz1;
  float    FLTx2;
  float    FLTx2_err;
  float    FLTy2;
  float    FLTy2_err;
  float    FLTz2;

  unsigned int nTriggers;           /* # of triggers using this track          */
  unsigned int nSlicerTrks;          /* # of Slicer tracks                      */
  unsigned int nRefitTrks;          /* # of refit tracks                       */
  unsigned int nMagTrks;            /* # of magnet tracks                      */
  unsigned int nSiliTrks;           /* # of Sili tracks                        */

  /* This is followed by variable length structures - see below */

};
typedef struct L2IO_TRACK L2IO_TRACK;

/* Extensions for L2IO_TRACK: */

struct L2IO_TRACK_TRIGGER {
  int TriggerIndex; /* Index of triggers using this track      */
};
typedef struct L2IO_TRACK_TRIGGER L2IO_TRACK_TRIGGER;

struct L2IO_TRACK_REFIT {
  int TrkNumber;         /* Refit track number */
  int nTracks;           /* # of sub tracks */
  int parent;         /* Number of parent slicer track */
};
typedef struct L2IO_TRACK_REFIT L2IO_TRACK_REFIT ;

struct L2IO_TRACK_MAGNET {                 /* Magnet track */
   int TrkNumber;                        /* Magnet track number */
   int Refpar;                           /* Refit parent */
   int RefSubpar;                        /* Refit subparent */
   int nTracks;                          /* n. of subtracks */
};
typedef struct L2IO_TRACK_MAGNET L2IO_TRACK_MAGNET;

struct L2IO_TRACK_SILI {       /* Silicon track */
  int TrkNumber;               /* Silicon track number */
  int nTracks;                 /* # of sub tracks */
  int pMagnetTrk;              /* Corresponding magnet track */
  int pMagSubTrk;              /* Corresponding magnet subtrack */
};
typedef struct L2IO_TRACK_SILI L2IO_TRACK_SILI;

/* End of extensions */

/* Definition of the different subtrack information blocks:*/


struct L2IO_SLICER {   
  int TrkNumber;         /* The first two is just to ensure correct reassembly */
  int SubTrkNumber;
  int trackID;      /* pretrigger track ID */
  float energy;          /* total energy reconstructed */
  float e_err;           /* error on energy, or 0. if unavailable */
  float pxpz;            /* Px/Pz */
  int   charge;            /* charge */
  
  float x1;              /* The RoI definition: */
  float x1_err;
  float y1;
  float y1_err;
  float z1;
  float x2;
  float x2_err;
  float y2;
  float y2_err;
  float z2;    
};
typedef struct L2IO_SLICER L2IO_SLICER;

struct L2IO_REFIT       {   /* ReFit Sub-track */
  int L2TrkNumber;       /* The first fields identify where to place the track */
  int RefitTrkNumber;    
  int    TrkNumber;         /* Refit sub-track number */
  int    nlayers;           /* # of single layers */
  int    nEmptyRoI;         /* # of empty layers */
  int    nUsed;             /* # of hits used */
  int    nHoles;            /* # of layers not used */
  int    nHolesInSeries;    /* # of holes in series */
  float  chi2sum;         /* pseudo chisq */
  float  quality;         /* track quality */
  float  intercept[2];    /* track intercept */
  float  slope[2];        /* track slope */
  float  xpos;            /* track's present x position */
  float  ypos;            /* track's present y position */
  float  zpos;            /* track's present z position */
  float  x_error;         /* track's present x position error*/
  float  y_error;         /* track's present y position error */
  float sw[2];           /* sum ( 1 / err_i^2 ) */
  float swz[2];          /* sum ( z_i / err_i^2 ) */
  float swzz[2];         /* sum ( z_i * z_i / err_i^2 ) */
  float swx[2];          /* sum ( x_i / err_i^2 ) */
  float swxz[2];         /* sum ( x_i * z_i / err_i^2 ) */
  float  delx;
  int    nHits;
  int pHit[REFIT_MAX_HIT];
};
typedef struct L2IO_REFIT L2IO_REFIT;

struct L2IO_REFIT2       {   /* ReFit Sub-track */
  int L2TrkNumber;       /* The first fields identify where to place the track */
  int RefitTrkNumber;    
  int    TrkNumber;         /* Refit sub-track number */
  int    nlayers;           /* # of single layers */
  int    nEmptyRoI;         /* # of empty layers */
  int    nUsed;             /* # of hits used */
  int    nHoles;            /* # of layers not used */
  int    nHolesInSeries;    /* # of holes in series */
  float  chi2sum;         /* pseudo chisq */
  float  quality;         /* track quality */
  float  intercept[2];    /* track intercept */
  float  slope[2];        /* track slope */
  float  xpos;            /* track's present x position */
  float  ypos;            /* track's present y position */
  float  zpos;            /* track's present z position */
  float  x_error;         /* track's present x position error*/
  float  y_error;         /* track's present y position error */
  float sw[2];           /* sum ( 1 / err_i^2 ) */
  float swz[2];          /* sum ( z_i / err_i^2 ) */
  float swzz[2];         /* sum ( z_i * z_i / err_i^2 ) */
  float swx[2];          /* sum ( x_i / err_i^2 ) */
  float swxz[2];         /* sum ( x_i * z_i / err_i^2 ) */
  float  delx;
  float richradius;          /* Rich information */
  float richtx;              /* from analysis package */
  float richty;
  int    nHits;
  int pHit[REFIT_MAX_HIT];
};
typedef struct L2IO_REFIT2 L2IO_REFIT2;

struct L2IO_Magnet {   /* Magnet subtrack */
  int L2TrkNumber;       /* The first fields identify where to place the track */
  int MagnetTrkNumber;    
  int TrkNumber;          /* Magnet sub track number */
  float  slopexz;          /* track's slope xz plane*/
  float  slopeyz;          /* track's slope yz plane*/
  float  invmom;           /* track's inverse momentum (1/pz)*charge */
  float  xpos;             /* track's x position */
  float  ypos;             /* track's y position */
  float error[5][5];      /* parameters error matrix */
  float  zpos;             /* track's z position */
  float  chisq;            /* current chisquare estimate */
  int nHits;
  int pHit[MAGNET_MAX_HIT];
};
typedef struct L2IO_Magnet L2IO_MAGNET;

struct L2IO_SILI { /* Silicon sub track */
  int   L2TrkNumber; /* The first fields identify where to place the track */
  int   SiliTrkNumber;    
  int   TrkNumber;   /* Silicon sub track number */
  float MOM   ;    /*  momemtum of track candidate */
  float SLO[2];    /*  current slope estimate */
  float INT[2];    /*  current intercept estimate */
  float S0[2];     /*  sum of weights */
  float SZ[2];     /*  sum of weighted z */
  float SZSQ[2];   /*  sum of weighted z**2 */
  float RX[2];     /*  sum of weighted x */
  float RXZ[2];    /*  sum of weighted x*z */
  float OS[2];     /*  sum of weigthed opposite direction cosines */
  float OZ[2];     /*  sum of weigthed z * opposite direction cosines */
  float OZSQ[2];   /*  sum of weighted z**2 * opposite direction cosines */
  int   NHIT[2];   /*  number of hits on track */
  int   NLTR[2];   /*  number of layers traversed in the last super plane */
  float CHI[2];    /*  current chisq estimate */
  
  int nHits;
  int pHit[SILI_MAX_HIT];
};
typedef struct L2IO_SILI L2IO_SILI;

struct L2IO_SILI2 { /* Silicon sub track */
  int   L2TrkNumber; /* The first fields identify where to place the track */
  int   SiliTrkNumber;    
  int   TrkNumber;   /* Silicon sub track number */
  float MOM   ;    /*  momemtum of track candidate */
  float SLO[2];    /*  current slope estimate */
  float INT[2];    /*  current intercept estimate */
  float S0[2];     /*  sum of weights */
  float SZ[2];     /*  sum of weighted z */
  float SZSQ[2];   /*  sum of weighted z**2 */
  float RX[2];     /*  sum of weighted x */
  float RXZ[2];    /*  sum of weighted x*z */
  float OS[2];     /*  sum of weigthed opposite direction cosines */
  float OZ[2];     /*  sum of weigthed z * opposite direction cosines */
  float OZSQ[2];   /*  sum of weighted z**2 * opposite direction cosines */
  int   NHIT[2];   /*  number of hits on track */
  int   NLTR[2];   /*  number of layers traversed in the last super plane */
  float CHI[2];    /*  current chisq estimate */
  int status_wire;  /* Combined status and wire fields */
  float impact;              /* Impact parameter to wire */
  float alongwire;           /* distance along the the wire */

  int nClustsBrems;          /* How many clusters did we find? */

  float Ebrems;              /* Bremsstrahlung information from the */
  float xbrems;              /* analysis package */
  float ybrems;

  int nHits;
  int pHit[SILI_MAX_HIT];
};
typedef struct L2IO_SILI2 L2IO_SILI2;



struct L2IO_HIT_HEAD {
  int TrkNumber;
  int nTotalHits;       /* The total number of hits for this track */
  int nHits;            /* Number of hits in this package. */
  int FirstHitNumber;   /* Number of the first hit */
};
typedef struct L2IO_HIT_HEAD L2IO_HIT_HEAD;

#define L2IO_MAXHITS 20  /* Currently arbitrarily chosen */

struct L2IO_HIT_BODY {
  int   plane;
  int   sl_rel;   /*PC1,MS10 = 0, PC2,MS11 = 1....TC2*/
  float rot;    /* hit coordinate */
  float offset; /* drift distance (or 0) */
  float res;
  float z;
  /* no PLANE information saved. Needed? */
};

typedef struct L2IO_HIT_BODY L2IO_HIT_BODY;


/* Handler for output of SLT summary to data stream (EVA) */
int L2IO_Handler(int event_slot, ETB_out_event *evt);

/* Routines to assemble L2 information */
int L2IO_InitData(int mask,L2EVENT *pEvent);
int L2IO_GetNext(L2EVENT *pEvent,char *buff,int *len,
		 int maxlen,int *compid,int *rev);


/*-----------------------   Target Information   --------------------------------*/

/* The following should not be here?? Brian */

#define L2IO_TARG_VERS      0

struct TARGET {
  long  time;                /* Database time stamp for target update */
  float InterActionRate;     /* Guess what... */
  float BackgroundRate;
  float TargetPosition[8];
};
typedef struct TARGET TARGET;

struct L2IO_TARG {
  unsigned int magic;
  TARGET target;
};
typedef struct L2IO_TARG L2IO_TARG;

/* Handler for output of target information to event stream (EVA) */
int L2IO_HandleTarget(int event_slot, ETB_out_event *evt);





#endif

