/* l2io_reader.c - Simple test program that reads L2IO output from a file */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "etb.h"
#include "l2io_read.h"

static int l2io_fd = -1;
static char* fifo;
#define MAXTRIES 3
#define NAPTIME 5

static void L2IO_ReadInit(L2EVENT *l2event);
static void L2IO_Read(L2EVENT *l2event,char *buff,int cmpid,int rev);
static void L2IO_ReadEvent(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadEventAux(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadTrigger(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadMainTrack(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadSlicerTrack(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadRefitTrack(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadMagnetTrack(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadSiliTrack(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadSlicerHit(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadRefitHit(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadMagnetHit(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadSiliHit(L2EVENT *l2event,char *buff,int rev);
static void L2IO_ReadHit(L2HIT *l2hit, char *buff);

int ReadEvent(ETB_fed *feds,L2EVENT *l2event, int num);

int ReadEvent(ETB_fed *feds,L2EVENT *l2event, int num)
{
  int i;
  char buff[1000];

  L2IO_ReadInit(l2event);
  for(i=0;i<num;i++) {
    ESP_get_int ( (int *) feds[i].data, (int *) buff,(feds[i].hdr.length-sizeof(SLB_SUBHDR))/4);
    L2IO_Read(l2event,buff,feds[i].hdr.fedid>>8,feds[i].hdr.recid-L2IO_FED_RECORD_ID);
  }
  return 1;
}
 

static void L2IOPipestart() {
  int fd, tries;
  char* slt_dom;
  char tmpstr[80];

/*   printf("entering L2IOstart\n"); */

  l2io_fd=-1;
  fd=-1;

  slt_dom = getenv("SLT_DOM");

  if (slt_dom==NULL) 
   {
    printf("Couldn't get environment variable SLT_DOM\n");
    return;
   }
  if (slt_dom[0]=='/')  slt_dom = slt_dom + 1;
  if (slt_dom[strlen(slt_dom)-1]=='/')  slt_dom[strlen(slt_dom)-1]='\0';
  sprintf(tmpstr,"/tmp/fifo_%s",slt_dom);
  fifo = tmpstr;

/*   printf("L2IOstart: fifo name = %s\n",fifo); */

  for (tries=1; tries <= MAXTRIES; tries++)
    {
     if ((fd=open(fifo, O_RDONLY|O_NDELAY))!=-1) break;
     sleep(NAPTIME);
    }
  if (fd==-1)
   {
    printf("Couldn't open fifo %s\n",fifo);
    return;
   }
  if (fcntl(fd,F_SETFL,O_RDONLY)==-1)
   {
    printf("Couldn't set permissions on fifo %s\n",fifo);
    return;
   }
  l2io_fd = fd;
}

void L2IOPipeEvent(L2EVENT *l2event) {
  char buff[1000];
  int size;
   struct {
    int cmpid;
    int rev;
    int len;
  } head;
 int done=0;
 L2IO_ReadInit(l2event);

 if (l2io_fd==-1) L2IOPipestart();

 /* Read the sltinfo */
 while(!done) {
   while ((size=read(l2io_fd,&head,sizeof(head)))==0)
     sleep(NAPTIME);
   if(size!=sizeof(head)) {
     printf("Only read %d - assuming end of file reached\n",size);
     return;
   }
   ESP_get_int ( (int *) &head, (int *) &head, sizeof(head)/4);
   if (head.len==0) done=1;
   else {
     while ((size=read(l2io_fd,buff,head.len))==0)
       sleep(NAPTIME);
     if (size!=head.len)
       printf("Warning only read %d bytes out of %d\n",size,head.len);
     ESP_get_int ( (int *) buff, (int *) buff, head.len/4);
     L2IO_Read(l2event,buff,head.cmpid,head.rev);
   }
 }
 return;
}


 
static void L2IO_ReadInit(L2EVENT *l2event) {
  int i;
  int *ptr;

  ptr = (int *) l2event;

  /* For now we just clear the whole structure... */

  for( i=0 ; i<sizeof(L2EVENT)/4 ; i++ )
    ptr[i] = 0;
  return;
}

/* There is currently practically no sanity checks on the input! */
static void L2IO_Read(L2EVENT *l2event,char *buff,int cmpid,int rev) {
  switch(cmpid) {
  case L2COMP_EVENT:
    L2IO_ReadEvent(l2event,buff,rev);
    break;
  case L2COMP_EVENT_AUX:
    L2IO_ReadEventAux(l2event,buff,rev);
    break;
  case L2COMP_TRIG:
    L2IO_ReadTrigger(l2event,buff,rev);
    break;
  case L2COMP_TRACK:
    L2IO_ReadMainTrack(l2event,buff,rev);
    break;
  case L2COMP_SLICER:
    L2IO_ReadSlicerTrack(l2event,buff,rev);
    break;
  case L2COMP_REFIT:
    L2IO_ReadRefitTrack(l2event,buff,rev);
    break;
  case L2COMP_MAGNET:
    L2IO_ReadMagnetTrack(l2event,buff,rev);
    break;
  case L2COMP_SILI:
    L2IO_ReadSiliTrack(l2event,buff,rev);
    break;
  case L2COMP_SLICER_HIT:
    L2IO_ReadSlicerHit(l2event,buff,rev);
    break;
  case L2COMP_REFIT_HIT:
    L2IO_ReadRefitHit(l2event,buff,rev);
    break;
  case L2COMP_MAGNET_HIT:
    L2IO_ReadMagnetHit(l2event,buff,rev);
    break;
  case L2COMP_SILI_HIT:
    L2IO_ReadSiliHit(l2event,buff,rev);
    break;
  default:
    printf("Unknown L2IO CompID - skipping... \n");
  }
}

static void L2IO_ReadEvent(L2EVENT *l2event,char *buff,int rev) {

  L2IO_EVENT *hdr = (L2IO_EVENT *) buff;

  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_EVENT\n");
    return;
  }

  if (hdr->magic!=L2IO_MAGIC) {
    printf("Bad magic marker in L2IO_EVENT\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_EVENT\n");
#endif

  l2event->eventType  = hdr->eventType;
  l2event->nTriggers  = hdr->nTriggers;
  l2event->nTracks    = hdr->nTracks;
  l2event->algo       = hdr->algo;
  l2event->acceptMask = hdr->acceptMask;
  l2event->trigMask   = hdr->trigMask;
  l2event->compMask   = hdr->l2compMask;
  l2event->time       = hdr->time;

}


static void L2IO_ReadEventAux(L2EVENT *l2event,char *buff,int rev) {

  L2IO_EVENT_AUX *aux = (L2IO_EVENT_AUX *) buff;
  int i;

  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_EVENT_AUX\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_EVENT_AUX\n");
#endif

  l2event->geo=aux->geo;     /* Not sure this is the correct way */

  l2event->nFLTTracks = aux->nFLTTracks;
  for ( i=0 ; i < aux->nFLTTracks ; i++ )
    l2event->SLTTrack[i+1] = aux->SLTTrack[i];

}

static void L2IO_ReadTrigger(L2EVENT *l2event,char *buff,int rev) {

  L2IO_TRIG *l2trg = (L2IO_TRIG *) buff;
  L2IO_TRIG2 *l2trg2 = (L2IO_TRIG2 *) buff;
  L2TRIGGER *trg;
  L2VERTEX *vert;
  int i;

  /* Do we know how to read this data? */
  if (rev!=L2IO_TRIG_VERS&&rev!=L2IO_TRIG_VERS2) {
    printf("Unknown revision of L2IO_TRIG\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_TRIG\n");
#endif

  if (rev==L2IO_REFIT_VERS) {

    trg=l2event->Triggers+l2trg->TriggerNumber;
    vert=&trg->vertex;

    trg->TriggerNumber = l2trg->TriggerNumber;
    trg->status        = l2trg->status;
    trg->trigID        = l2trg->trigID;
    trg->nTrks         = l2trg->nTrks;
    trg->flag          = 0;
    vert->chisq        = -1;
    vert->xv           = 0.;
    vert->yv           = 0.;
    vert->zv           = 0.;
    vert->sigxv        = 0.;
    vert->sigyv        = 0.;
    vert->sigzv        = 0.;
    vert->dxy          = 0.;
    vert->wire         = 0.;
    vert->dz           = 0.;
    vert->impact       = 0.;
    vert->alongwire    = 0.;
    vert->trks[0]      = NULL;
    vert->trks[1]      = NULL;
    

    for ( i=0 ; i < l2trg->nTrks ; i++ )
      trg->TrkIndex[i] = l2trg->TrkIndex[i];
  }
  else {

    trg=l2event->Triggers+l2trg2->TriggerNumber;
    vert=&trg->vertex;


    trg->TriggerNumber = l2trg2->TriggerNumber;
    trg->status        = l2trg2->status;
    trg->trigID        = l2trg2->trigID;
    trg->nTrks         = l2trg2->nTrks;

    trg->flag          = l2trg2->flag;
    vert->chisq        = l2trg2->chisq;
    vert->xv           = l2trg2->xv;
    vert->yv           = l2trg2->yv;
    vert->zv           = l2trg2->zv;
    vert->sigxv        = l2trg2->sigxv;
    vert->sigyv        = l2trg2->sigyv;
    vert->sigzv        = l2trg2->sigzv;
    vert->dxy          = l2trg2->dxy;
    vert->wire         = l2trg2->wire;
    vert->dz           = l2trg2->dz;
    vert->impact       = l2trg2->impact;
    vert->alongwire    = l2trg2->alongwire;

    for ( i=0 ; i < l2trg2->nTrks ; i++ ) {
      L2TRACK *pTrk;

      L2SILITRK *pSiliTrk;
      trg->TrkIndex[i] = l2trg2->TrkIndex[i];
      if (l2trg2->trks[i]==-1) {
	pTrk=&l2event->Tracks[l2trg2->TrkIndex[i]];
	pSiliTrk=&pTrk->SiliTrkSet.l2silitrk[l2trg2->trks[i]];
	vert->trks[i]=&(pSiliTrk->SiliSubTrk[l2trg2->subtrks[i]]);
      }
      else {
	vert->trks[i]=NULL;
      }
    }
  }
    
}

static void L2IO_ReadMainTrack(L2EVENT *l2event,char *buff,int rev) {

  L2IO_TRACK *trk = (L2IO_TRACK *) buff;
  L2IO_TRACK_TRIGGER *trktrig = (L2IO_TRACK_TRIGGER *) (buff+sizeof(L2IO_TRACK));
  L2IO_TRACK_REFIT *trkrefit;
  L2IO_TRACK_MAGNET *trkmagnet;
  L2IO_TRACK_SILI *trksili;
  L2TRACK *l2trk;
  int i;

  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_TRACK\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_TRACK\n");
#endif
  
  l2trk=l2event->Tracks+trk->TrkNumber;

  l2trk->TrkNumber  = trk->TrkNumber;
  l2trk->FLTNumber  = trk->FLTNumber;
  l2trk->status     = trk->status;
  l2trk->info       = trk->info;

  l2trk->FLTTrk.trackID = trk->FLTtrackID;
  l2trk->FLTTrk.energy  = trk->FLTenergy;
  l2trk->FLTTrk.e_err   = trk->FLTe_err;
  l2trk->FLTTrk.pxpz    = trk->FLTpxpz;
  l2trk->FLTTrk.charge  = trk->FLTcharge;
  l2trk->FLTTrk.x1      = trk->FLTx1;
  l2trk->FLTTrk.x1_err  = trk->FLTx1_err;
  l2trk->FLTTrk.y1      = trk->FLTy1;
  l2trk->FLTTrk.y1_err  = trk->FLTy1_err;
  l2trk->FLTTrk.z1      = trk->FLTz1;
  l2trk->FLTTrk.x2      = trk->FLTx2;
  l2trk->FLTTrk.x2_err  = trk->FLTx2_err;
  l2trk->FLTTrk.y2      = trk->FLTy2;
  l2trk->FLTTrk.y2_err  = trk->FLTy2_err;
  l2trk->FLTTrk.z2      = trk->FLTz2;

  l2trk->nTriggers  = trk->nTriggers;
  l2trk->L2SlicerTrkSet.nTracks = trk->nSlicerTrks;
  l2trk->ReFitTrkSet.nTracks    = trk->nRefitTrks;
  l2trk->MagnetTrkSet.nTracks   = trk->nMagTrks;
  l2trk->SiliTrkSet.nTracks     = trk->nSiliTrks;

  /* Copy trigger references */
  for ( i=0 ; i < l2trk->nTriggers ; i++ ) {
    l2trk->TriggerIndex[i] = trktrig->TriggerIndex;
    trktrig++;
  }

  /* Copy Refit tracks */
  trkrefit=(L2IO_TRACK_REFIT *) trktrig;

  for ( i=0 ; i < trk->nRefitTrks ; i++) {
    l2trk->ReFitTrkSet.l2refittrk[i].TrkNumber = trkrefit->TrkNumber;
    l2trk->ReFitTrkSet.l2refittrk[i].nTracks = trkrefit->nTracks;
    l2trk->ReFitTrkSet.l2refittrk[i].parent = 
      l2trk->L2SlicerTrkSet.l2SlicerTrk+trkrefit->parent;
    trkrefit++;
  }
  /* Copy Magnet tracks */

  trkmagnet=(L2IO_TRACK_MAGNET *) trkrefit;

  for( i=0 ; i < trk->nMagTrks ; i++) {
    l2trk->MagnetTrkSet.L2magnetTrk[i].TrkNumber = trkmagnet->TrkNumber;
    l2trk->MagnetTrkSet.L2magnetTrk[i].Refpar    = trkmagnet->Refpar;
    l2trk->MagnetTrkSet.L2magnetTrk[i].RefSubpar = trkmagnet->RefSubpar;
    l2trk->MagnetTrkSet.L2magnetTrk[i].nTracks  =  trkmagnet->nTracks;
    trkmagnet++;
  }

  /* Copy silicon tracks */
  
  trksili=(L2IO_TRACK_SILI *) trkmagnet;
  
  for( i=0 ; i < trk->nSiliTrks ; i++ ) {
    l2trk->SiliTrkSet.l2silitrk[i].TrkNumber  = trksili->TrkNumber;
    l2trk->SiliTrkSet.l2silitrk[i].nTracks    = trksili->nTracks;
    l2trk->SiliTrkSet.l2silitrk[i].pMagnetTrk = 
      l2trk->MagnetTrkSet.L2magnetTrk+trksili->pMagnetTrk;
    l2trk->SiliTrkSet.l2silitrk[i].pMagSubTrk =
    l2trk->MagnetTrkSet.L2magnetTrk[trksili->pMagnetTrk].L2MagnetSubTrk+
      trksili->pMagSubTrk;
    trksili++;
  }
}


static void L2IO_ReadSlicerTrack(L2EVENT *l2event,char *buff,int rev) {

  L2IO_SLICER *trk = (L2IO_SLICER *) buff;
  L2TRACK *l2trk;

  int SlicerTrk;

  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_SLICER\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_SLICER\n");
#endif

  l2trk = l2event->Tracks+trk->TrkNumber;

  SlicerTrk = trk->SubTrkNumber;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].trackID = trk->trackID;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].energy  = trk->energy;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].e_err   = trk->e_err;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].pxpz    = trk->pxpz;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].charge  = trk->charge;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].x1      = trk->x1;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].x1_err  = trk->x1_err;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].y1      = trk->y1;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].y1_err  = trk->y1_err;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].z1      = trk->z1;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].x2      = trk->x2;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].x2_err  = trk->x2_err;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].y2      = trk->y2;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].y2_err  = trk->y2_err;
  l2trk->L2SlicerTrkSet.l2SlicerTrk[SlicerTrk].z2      = trk->z2;
}


static void L2IO_ReadRefitTrack(L2EVENT *l2event,char *buff,int rev) {

  L2IO_REFIT *trk = (L2IO_REFIT *) buff;
  L2IO_REFIT2 *trk2 = (L2IO_REFIT2 *) buff;
  L2REFITSUBTRK *subtrk;
  L2TRACK *l2trk;
  L2HIT *firstHit;

  int i;
  int RefitTrk,RefitSubTrk;

  /* Do we know how to read this data? */
  if (rev!=L2IO_REFIT_VERS&&rev!=L2IO_REFIT_VERS2) {
    printf("Unknown revision of L2IO_REFIT %d\n",rev);
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_REFIT\n");
#endif

  if (rev==L2IO_REFIT_VERS) {

    l2trk = l2event->Tracks+trk->L2TrkNumber;
    
    RefitTrk = trk->RefitTrkNumber;

    RefitSubTrk=trk->TrkNumber;

    subtrk=&(l2trk->ReFitTrkSet.l2refittrk[RefitTrk].RefitSubTrk[RefitSubTrk]);

    subtrk->TrkNumber     =trk->TrkNumber;     
    subtrk->nlayers       =trk->nlayers;       
    subtrk->nEmptyRoI     =trk->nEmptyRoI;     
    subtrk->nUsed         =trk->nUsed;         
    subtrk->nHoles        =trk->nHoles;        
    subtrk->nHolesInSeries=trk->nHolesInSeries;
    subtrk->chi2sum       =trk->chi2sum;       
    subtrk->quality       =trk->quality;       
    subtrk->intercept[0]  =trk->intercept[0];  
    subtrk->intercept[1]  =trk->intercept[1];  
    subtrk->slope[0]      =trk->slope[0];      
    subtrk->slope[1]      =trk->slope[1];      
    subtrk->xpos          =trk->xpos;          
    subtrk->ypos          =trk->ypos;          
    subtrk->zpos          =trk->zpos;          
    subtrk->x_error       =trk->x_error;       
    subtrk->y_error       =trk->y_error;       
    subtrk->sw[0]         =trk->sw[0];         
    subtrk->sw[1]         =trk->sw[1];         
    subtrk->swz[0]        =trk->swz[0];        
    subtrk->swz[1]        =trk->swz[1];        
    subtrk->swzz[0]       =trk->swzz[0];       
    subtrk->swzz[1]       =trk->swzz[1];       
    subtrk->swx[0]        =trk->swx[0];        
    subtrk->swx[1]        =trk->swx[1];        
    subtrk->swxz[0]       =trk->swxz[0];       
    subtrk->swxz[1]       =trk->swxz[1];       
    subtrk->delx          =trk->delx;          
    subtrk->nHits         =trk->nHits;         
    l2trk->ReFitTrkSet.l2refittrk[RefitTrk].richradius    =-1.0;
    l2trk->ReFitTrkSet.l2refittrk[RefitTrk].richtx        = 0.0;
    l2trk->ReFitTrkSet.l2refittrk[RefitTrk].richty        = 0.0;

    firstHit=l2trk->YBackHits.Hit;
    for( i=0 ; i < trk->nHits ; i++)
      subtrk->pHit[i]=trk->pHit[i]+firstHit;
  }
  else { /* This code duplications, but as the structures are changed it is
            cleaner this way */
    l2trk = l2event->Tracks+trk2->L2TrkNumber;
    
    RefitTrk = trk2->RefitTrkNumber;

    RefitSubTrk=trk2->TrkNumber;

    subtrk=&(l2trk->ReFitTrkSet.l2refittrk[RefitTrk].RefitSubTrk[RefitSubTrk]);

    subtrk->TrkNumber     =trk2->TrkNumber;     
    subtrk->nlayers       =trk2->nlayers;       
    subtrk->nEmptyRoI     =trk2->nEmptyRoI;     
    subtrk->nUsed         =trk2->nUsed;         
    subtrk->nHoles        =trk2->nHoles;        
    subtrk->nHolesInSeries=trk2->nHolesInSeries;
    subtrk->chi2sum       =trk2->chi2sum;       
    subtrk->quality       =trk2->quality;       
    subtrk->intercept[0]  =trk2->intercept[0];  
    subtrk->intercept[1]  =trk2->intercept[1];  
    subtrk->slope[0]      =trk2->slope[0];      
    subtrk->slope[1]      =trk2->slope[1];      
    subtrk->xpos          =trk2->xpos;          
    subtrk->ypos          =trk2->ypos;          
    subtrk->zpos          =trk2->zpos;          
    subtrk->x_error       =trk2->x_error;       
    subtrk->y_error       =trk2->y_error;       
    subtrk->sw[0]         =trk2->sw[0];         
    subtrk->sw[1]         =trk2->sw[1];         
    subtrk->swz[0]        =trk2->swz[0];        
    subtrk->swz[1]        =trk2->swz[1];        
    subtrk->swzz[0]       =trk2->swzz[0];       
    subtrk->swzz[1]       =trk2->swzz[1];       
    subtrk->swx[0]        =trk2->swx[0];        
    subtrk->swx[1]        =trk2->swx[1];        
    subtrk->swxz[0]       =trk2->swxz[0];       
    subtrk->swxz[1]       =trk2->swxz[1];       
    subtrk->delx          =trk2->delx;         
    l2trk->ReFitTrkSet.l2refittrk[RefitTrk].richradius    =trk2->richradius;
    l2trk->ReFitTrkSet.l2refittrk[RefitTrk].richtx        =trk2->richtx;
    l2trk->ReFitTrkSet.l2refittrk[RefitTrk].richty        =trk2->richty;
    subtrk->nHits         =trk2->nHits;         

    firstHit=l2trk->YBackHits.Hit;
    for( i=0 ; i < trk2->nHits ; i++)
      subtrk->pHit[i]=trk2->pHit[i]+firstHit;
  }
}


static void L2IO_ReadMagnetTrack(L2EVENT *l2event,char *buff,int rev) {

  L2IO_MAGNET *trk = (L2IO_MAGNET *) buff;
  L2MAGNETSUBTRK *subtrk;
  L2TRACK *l2trk;
  L2HIT *firstHit;

  int i,j;
  int MagnetTrk,MagnetSubTrk;

  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_MAGNET\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_MAGNET\n");
#endif

  l2trk = l2event->Tracks+trk->L2TrkNumber;

  MagnetTrk = trk->MagnetTrkNumber;

  MagnetSubTrk=trk->TrkNumber;

  subtrk=&(l2trk->MagnetTrkSet.L2magnetTrk[MagnetTrk].L2MagnetSubTrk[MagnetSubTrk]);


  subtrk->TrkNumber     =trk->TrkNumber;     
  subtrk->slopexz = trk->slopexz;
  subtrk->slopeyz = trk->slopeyz;
  subtrk->invmom  = trk->invmom; 
  subtrk->xpos    = trk->xpos;   
  subtrk->ypos    = trk->ypos;   
  for(i=0;i<5;i++)
    for(j=0;j<5;j++)
      subtrk->error[i][j] = trk->error[i][j];

  subtrk->zpos    = trk->zpos;   
  subtrk->chisq   = trk->chisq;  

  subtrk->nHits   = trk->nHits;         

  firstHit=l2trk->MagHits.Hit;
  for( i=0 ; i < trk->nHits ; i++)
    subtrk->pHit[i]=trk->pHit[i]+firstHit;
}


static void L2IO_ReadSiliTrack(L2EVENT *l2event,char *buff,int rev) {

  L2IO_SILI *trk = (L2IO_SILI *) buff;
  L2IO_SILI2 *trk2 = (L2IO_SILI2 *) buff;
  L2SILISUBTRK *subtrk;
  L2TRACK *l2trk;
  L2HIT *firstHit;

  int i;
  int SiliTrk,SiliSubTrk;

  /* Do we know how to read this data? */
  if (rev!=L2IO_SILI_VERS&&rev!=L2IO_SILI_VERS2) {
    printf("Unknown revision of L2IO_SILI\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_SILI\n");
#endif

  if (rev==L2IO_SILI_VERS) {

    l2trk = l2event->Tracks+trk->L2TrkNumber;

    SiliTrk = trk->SiliTrkNumber;
    
    SiliSubTrk = trk->TrkNumber;

    subtrk=&(l2trk->SiliTrkSet.l2silitrk[SiliTrk].SiliSubTrk[SiliSubTrk]);

    subtrk->MOM     = trk->MOM   ; 
    for(i=0;i<2;i++) {
      subtrk->SLO[i]  = trk->SLO[i]; 
      subtrk->INT[i]  = trk->INT[i]; 
      subtrk->S0[i]   = trk->S0[i];  
      subtrk->SZ[i]   = trk->SZ[i];  
      subtrk->SZSQ[i] = trk->SZSQ[i];
      subtrk->RX[i]   = trk->RX[i];  
      subtrk->RXZ[i]  = trk->RXZ[i]; 
      subtrk->OS[i]   = trk->OS[i];  
      subtrk->OZ[i]   = trk->OZ[i];  
      subtrk->OZSQ[i] = trk->OZSQ[i];
      subtrk->NHIT[i] = trk->NHIT[i];
      subtrk->NLTR[i] = trk->NLTR[i];
      subtrk->CHI[i]  = trk->CHI[i]; 
    }
    subtrk->nHits         =trk->nHits;         
  /*  printf("hits %d\n",trk->nHits);*/

    subtrk->status    =L2SILISUB_GOOD;
    subtrk->wirenr      =-1;
    subtrk->impact        = 0.;
    subtrk->alongwire     = 0.;
    subtrk->nClustsBrems  = -1;
    subtrk->Ebrems        = 0.;
    subtrk->xbrems        = 0.;
    subtrk->ybrems        = 0.;


    firstHit=l2trk->SiliHits.Hit;
    for( i=0 ; i < trk->nHits ; i++)
      subtrk->pHit[i]=trk->pHit[i]+firstHit; 
  }
  else {

    l2trk = l2event->Tracks+trk2->L2TrkNumber;

    SiliTrk = trk2->SiliTrkNumber;
    
    SiliSubTrk = trk2->TrkNumber;

    subtrk=&(l2trk->SiliTrkSet.l2silitrk[SiliTrk].SiliSubTrk[SiliSubTrk]);

    subtrk->MOM     = trk2->MOM   ; 
    for(i=0;i<2;i++) {
      subtrk->SLO[i]  = trk2->SLO[i]; 
      subtrk->INT[i]  = trk2->INT[i]; 
      subtrk->S0[i]   = trk2->S0[i];  
      subtrk->SZ[i]   = trk2->SZ[i];  
      subtrk->SZSQ[i] = trk2->SZSQ[i];
      subtrk->RX[i]   = trk2->RX[i];  
      subtrk->RXZ[i]  = trk2->RXZ[i]; 
      subtrk->OS[i]   = trk2->OS[i];  
      subtrk->OZ[i]   = trk2->OZ[i];  
      subtrk->OZSQ[i] = trk2->OZSQ[i];
      subtrk->NHIT[i] = trk2->NHIT[i];
      subtrk->NLTR[i] = trk2->NLTR[i];
      subtrk->CHI[i]  = trk2->CHI[i]; 
    }
    subtrk->nHits     =trk2->nHits;         
    
    subtrk->status    =trk2->status_wire&0xf;
    subtrk->wirenr      =(trk2->status_wire&0xff0)/16;
    subtrk->impact        = trk2->impact;
    subtrk->alongwire     = trk2->alongwire;
    subtrk->nClustsBrems  = trk2->nClustsBrems;
    subtrk->Ebrems        = trk2->Ebrems;
    subtrk->xbrems        = trk2->xbrems;
    subtrk->ybrems        = trk2->ybrems;

    firstHit=l2trk->SiliHits.Hit;
    for( i=0 ; i < trk2->nHits ; i++)
      subtrk->pHit[i]=trk2->pHit[i]+firstHit; 
  }
}



static void L2IO_ReadSlicerHit(L2EVENT *l2event,char *buff,int rev) {
  L2IO_HIT_HEAD *head = (L2IO_HIT_HEAD *) buff;
  L2TRACK *l2trk;


  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_SLICER_Hit\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_SLICER_Hit\n");
#endif
  l2trk = l2event->Tracks+head->TrkNumber;
  l2trk->TrkHitSet.nHits=head->nTotalHits;

  L2IO_ReadHit(l2trk->TrkHitSet.Hit,buff);
}

static void L2IO_ReadRefitHit(L2EVENT *l2event,char *buff,int rev) {
  L2IO_HIT_HEAD *head = (L2IO_HIT_HEAD *) buff;
  L2TRACK *l2trk;


  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_REFIT_Hit\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_REFIT_Hit\n");
#endif
  l2trk = l2event->Tracks+head->TrkNumber;
  l2trk->YBackHits.nHits=head->nTotalHits;

  L2IO_ReadHit(l2trk->YBackHits.Hit,buff);
}

static void L2IO_ReadMagnetHit(L2EVENT *l2event,char *buff,int rev) {
  L2IO_HIT_HEAD *head = (L2IO_HIT_HEAD *) buff;
  L2TRACK *l2trk;


  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_MAGNET_Hit\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_MAGNET_Hit\n");
#endif

  l2trk = l2event->Tracks+head->TrkNumber;
  l2trk->MagHits.nHits=head->nTotalHits;

  L2IO_ReadHit(l2trk->MagHits.Hit,buff);
}

static void L2IO_ReadSiliHit(L2EVENT *l2event,char *buff,int rev) {
  L2IO_HIT_HEAD *head = (L2IO_HIT_HEAD *) buff;
  L2TRACK *l2trk;


  /* Do we know how to read this data? */
  if (rev!=0) {
    printf("Unknown revision of L2IO_SILI_Hit\n");
    return;
  }

#ifdef PRINT
  printf("Reading L2IO_SILI_Hit\n");
#endif

  l2trk = l2event->Tracks+head->TrkNumber;
  l2trk->SiliHits.nHits=head->nTotalHits;

  L2IO_ReadHit(l2trk->SiliHits.Hit,buff);
}

static void L2IO_ReadHit(L2HIT *l2hit, char *buff) {
  L2IO_HIT_HEAD *head = (L2IO_HIT_HEAD *) buff;
  L2IO_HIT_BODY *body = (L2IO_HIT_BODY *) (buff+sizeof(L2IO_HIT_HEAD));
  int i;

 for(i=head->FirstHitNumber;i<head->FirstHitNumber+head->nHits;i++) {
    l2hit[i].plane  = body->plane;
    l2hit[i].sl_rel = body->sl_rel;
    l2hit[i].rot    = body->rot;
    l2hit[i].offset = body->offset;
    l2hit[i].res    = body->res;
    l2hit[i].z 	    = body->z;
    body++;
  }
}


