#ifndef _TGS
#define _TGS

#include <stdio.h>

#include "slp.h"
#include "slp_error.h"
#include "rhp.h"
#include "l1data.h"
#include "component.h"

/***
   *
   * RHP prefixes for slp modules -- any which use the SLT histo group
   *
   ****/

enum {
  TPRE_HOT_LIST, /* Histos destined for data quality monitoring */
  TPRE_TGS,
  TPRE_SLT,
  TPRE_GFLT,
  TPRE_FLTE,
  TPRE_FLTI,
  TPRE_SLIC,
  TPRE_REFIT,
  TPRE_MAG,
  TPRE_SILI,
  TPRE_VERT,
  TPRE_TARG,
  TPRE_MAP,
  TPRE_OTR,
  TPRE_MUON,
  TPRE_PNA     
};


#define HOT_PRE(x)   (COMP_N_SLT << 24) | (TPRE_HOT_LIST << 12) | (x)
#define TGS_PRE(x)   (COMP_N_SLT << 24) | (TPRE_TGS  << 12) | (x)
#define SLT_PRE(x)   (COMP_N_SLT << 24) | (TPRE_SLT  << 12) | (x)
#define GFLT_PRE(x)  (COMP_N_SLT << 24) | (TPRE_GFLT << 12) | (x)
#define FLTE_PRE(x)  (COMP_N_SLT << 24) | (TPRE_FLTE  << 12) | (x)
#define FLTI_PRE(x)  (COMP_N_SLT << 24) | (TPRE_FLTI  << 12) | (x)
#define SLIC_PRE(x)  (COMP_N_SLT << 24) | (TPRE_SLIC  << 12) | (x)
#define REFIT_PRE(x) (COMP_N_SLT << 24) | (TPRE_REFIT << 12) | (x)
#define MAG_PRE(x)   (COMP_N_SLT << 24) | (TPRE_MAG   << 12) | (x)
#define SILI_PRE(x)  (COMP_N_SLT << 24) | (TPRE_SILI  << 12) | (x)
#define VERT_PRE(x)  (COMP_N_SLT << 24) | (TPRE_VERT  << 12) | (x)
#define TARG_PRE(x)  (COMP_N_SLT << 24) | (TPRE_TARG  << 12) | (x)
#define MAP_PRE(x)   (COMP_N_SLT << 24) | (TPRE_MAP   << 12) | (x)
#define OTR_PRE(x)   (COMP_N_SLT << 24) | (TPRE_OTR   << 12) | (x)
#define MUON_PRE(x)  (COMP_N_SLT << 24) | (TPRE_MUON  << 12) | (x)
#define PNA_PRE(x)   (COMP_N_SLT << 24) | (TPRE_PNA   << 12) | (x)

#define BOOK_N_CHECK1(id,title,nbins,lo,hi) \
{int rhp_rc = rhp_hbook1((id),(title),(nbins),(lo),(hi)); \
 if(rhp_rc != 0) {  char mess[100]; \
 sprintf(mess,"rhp is not pleased with %#x\n", (id)); \
 SLP_logError(rhp_rc, mess); }}

#define BOOK_N_CHECK2(id,title,nx,lox,hix,ny,loy,hiy) \
{int rhp_rc = rhp_hbook2((id),(title),(nx),(lox),(hix),(ny),(loy),(hiy)); \
 if(rhp_rc != 0) {  char mess[100]; \
 sprintf(mess,"rhp is not pleased with %#x\n", (id)); \
 SLP_logError(rhp_rc, mess); }}

int TGS_assignHotID(void); /* return a unique histo number, including prefixes */

/* 
   Scalers to keep track of the number of events passing cuts throughout the
   full SLT processing. The current scaler values are filled into  the "cuts"
   histogram and are kept locally. The values and titles can be obtained
   remotely via rpm messages of class SLT (to be implemented).
   */

int TGS_bookEventScaler(const char *title); /* return -1 on error, otherwise id */
void TGS_incrementScaler(int id);
void TGS_resetScalers();
void TGS_printScaler(FILE *fd, int id);
void TGS_printScalers(FILE *fd);

int TGS_startup(SLP_transitions which);
int TGS_initialize(SLP_transitions which);
int TGS_zero(SLP_transitions which);
int TGS_shutdown(SLP_transitions which);
void TGS_free(int event);

/* These are possible settings of "param" in TGS_trigger -- 
   allows for TGS to be called after all triggering is done, to update bxinator counters */

#define TGS_BX_INCREMENT_AND_TRIGGER 0 /* for param: increment counters in bxinator and trigger */
#define TGS_BX_INCREMENT_ONLY 1        /* just increment counts in bxinator, nothing else */
#define TGS_BX_TRIGGER_ONLY 2          /* don't increment counters, just trigger */

SLP_RETURN TGS_trigger(int event, int param);
SLP_RETURN TGS_dummie(int event, int param);

/***            Trigger and track banks and access functions               ***/

#define TGS_MAX_TRIGGERS 50
#define TGS_MAX_TRACKS 100

struct FLT_TRACK {  /* This is identical to L2ROI, for the moment */
  PARTTYPE trackID;      /* pretrigger track ID */
  float energy;          /* total energy reconstructed */
  float e_err;           /* error on energy, or 0. if unavailable */
  float pxpz;            /* Px/Pz */
  int charge;            /* charge */
  
  float x1;              /* The RoI definition: */
  float x1_err;
  float y1;
  float y1_err;
  float z1;
  float x2;
  float x2_err;
  float y2;
  float y2_err;
  float z2;
};
typedef struct FLT_TRACK FLT_TRACK;

int   TGS_addTrack(const FLT_TRACK *trk);          /* return track index or -1 on error   */
int   TGS_addTrigger(const L2FR_L1Trig *trig);     /* return trigger index or -1 on error */
const FLT_TRACK   *TGS_getTrack(int itrk);
const L2FR_L1Trig *TGS_getTrigger(int itrig);
const L2FR_L1Data *TGS_getHeader(void);

void  TGS_setTargetStretch(float stretch); /*** in cm. Target box is widened by this amount ***/
void  TGS_updateTargetStretch(void);
float TGS_getTargetStretch(void);
int   TGS_setUpstreamRoI( int itrk );
void  TGS_setHotBxVeto(int i);
void  TGS_setEmptyBxVeto(int i);

int   TGS_getNTracks(void);
int   TGS_getNTriggers(void);

void  TGS_printBanks(FILE *fd);
void  TGS_printHeader(FILE *fd);
void  TGS_printTracks(FILE *fd);
void  TGS_printTrack(FILE *fd, int itrk);
void  TGS_printTriggers(FILE *fd);
void  TGS_printTrigger(FILE *fd, int itrig);
const char *TGS_getMapOutFIle(); /* return name of file to be used for map printout -- for debugging */
void TGS_setActiveTrigger(int type_of_trigger);

#endif
